﻿--[[*******************************
	* Initilise all the variables *
	*****************************]]
-----------------------------------------
-- Get the mod info from the toc file. --
-----------------------------------------
local modName = select(1, ...)
local modVersion = GetAddOnMetadata(modName, "Version")
local modTitle = GetAddOnMetadata(modName, "Title")
local modDescription = GetAddOnMetadata(modName, "Notes")

-------------------------------
-- create all the stub items --
-------------------------------
local addon = LibStub("AceAddon-3.0"):NewAddon(modName, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigReg = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local L	= LibStub("AceLocale-3.0"):GetLocale(modName)
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")

---------------------------------------------------
-- get the global names and make them local ones --
---------------------------------------------------
local repFrames = {
	anchor = RepItemFrame,
	close = RepItemFrame_Close,
	query = RepItemFrame_Query,
	template = "RepItemTemplate",
	dropDownMenu = nil,
	interfacePage = nil,
	minimapButtonFrame = RepItemsButtonFrame,
	minimapButton = RepItemsButtonFrame_Button,
}

-------------------------------
-- set the default variables --
-------------------------------
local bankFrameOpen = false			-- boolean to show if bank frame is opened
local repItemButtons = {}			-- holds the buttons used by the frame
local frameIncreaseSize = 42		-- increase the frame size by this amount
local lineAdded = false
local playerName = ""				-- holds the player name
local realmName = ""				-- holds the realm name
local repMultiplier = 0				-- used for rep handin multiplier as humans get more than the rest
local buttonMoving = false			-- flag to indicate if the minimap button is being moved

------------------------------------------------------
-- create the defaults for the global save variable --
------------------------------------------------------
local defaults = {
	profile = {
		faction = "",
		ShowBankItems = true,
		ShowMinorItems = false,
		ShowOnlyWhenThereAreItems = false,
		ShowTooltips = true,
		ShowItemsFromOtherCharacters = true,
		ShowMinimapButton = false,
		MinimapButtonPosition = 350,
		frameClosedByUser = false,
		ShownValue = "QUANTITY",
		UIScale = 1.0,
		numberOfColumns = 4,
		itemsInBags = {},
	}
}

------------------------------
-- create the titan options --
------------------------------
local ldbOptions = {
	type = "launcher",
	icon = "Interface\\AddOns\\RepItems\\Images\\icon",
	label = L["Rep Items"],
	OnClick = function(clickedframe, button)
		if (button == "RightButton") then
			addon:OpenInterfacePage();
		else
			addon.db.profile.frameClosedByUser = not addon.db.profile.frameClosedByUser
			addon:ShowFrame(true)
			AceConfigReg:NotifyChange(modName)
		end
	end,
	OnTooltipShow = function(self) 
		self:AddLine(L["Rep Items"]) -- LDB Tooltip title
		self:AddLine(L["Left-click to toggle window"]) -- LDB click help: toggle
		self:AddLine(L["Right-click for options"]) -- LDB right-click help: options
	end,
}

--[[*****************
	* Handle events *
	***************]]
----------------
-- Ace3 calls --
----------------
function addon:OnInitialize()
	-- setup or get the saved variables
	addon.db = LibStub("AceDB-3.0"):New("RepItemsDB", defaults)
	
	-- register the config page
	AceConfig:RegisterOptionsTable(modName, addon.interfaceOptions)
	repFrames.interfacePage = AceConfigDialog:AddToBlizOptions(modName, "Rep Items")
	repFrames.interfacePage.default = function() addon:ResetToDefaults(); end

	-- register the slash command	
	addon:RegisterChatCommand("repitems", "SlashCommandHandler")

	-- hook the close button script
	addon:HookScript(repFrames.close, "OnClick")
	
	-- setup the drop down menu and mouse click
	repFrames.dropDownMenu = CreateFrame("Frame", "RepFrames.dropDownMenu", nil, "UIDropDownMenuTemplate")
	addon:HookScript(repFrames.anchor, "OnMouseUp")
	
	-- get the player info
	addon.db.profile.faction = select(1, UnitFactionGroup("player"))
	playerName = UnitName("player")
	realmName = GetRealmName()
	if (select(2, UnitRace("player")) == "Human") then
		repMultiplier = 10
	end
	
	-- hook the mouseover items when looking in bags or pickup items
	addon:SecureHookScript(GameTooltip, "OnTooltipSetItem")
	addon:SecureHookScript(GameTooltip, "OnTooltipCleared")
	
	-- add the minimap button and hook it
	if (addon.db.profile.ShowMinimapButton) then
		repFrames.minimapButtonFrame:Show()
	else
		repFrames.minimapButtonFrame:Hide()
	end
	addon:HookScript(repFrames.minimapButton, "OnDragStart")
	addon:HookScript(repFrames.minimapButton, "OnDragStop")
	addon:HookScript(repFrames.minimapButton, "OnUpdate")
	addon:HookScript(repFrames.minimapButton, "OnMouseUp")
	addon:HookScript(repFrames.minimapButton, "OnEnter")
	addon:HookScript(repFrames.minimapButton, "OnLeave")
	
	addon:HookScript(repFrames.query, "OnEnter")
	addon:HookScript(repFrames.query, "OnLeave")
	
	-- add menu to titan or fubar via libdatabroker (only tested with titan)
	local launcher = LDB:NewDataObject(modName, ldbOptions)
end

function addon:OnEnable()
	-- Register the calls that happen when the addon is enabled
	addon:RegisterEvent("BAG_UPDATE")	
	addon:RegisterEvent("BANKFRAME_OPENED")
	addon:RegisterEvent("BANKFRAME_CLOSED")
	
	-- and update the info to display
	addon:UpdateButtonPosition()

	-- show the welcome message
	addon:Print(L["Welcome"])
end

function addon:OnDisable()
	-- Unregister the calls that i dont want so that it does not take up resources
	addon:UnregisterEvent("BAG_UPDATE")	
	addon:UnregisterEvent("BANKFRAME_OPENED")
	addon:UnregisterEvent("BANKFRAME_CLOSED")
	
	-- hide the frame if not already
	repFrames.anchor:Hide()
end

------------------------------------------------
-- handle all the other calls from the system --
------------------------------------------------
function addon:BAG_UPDATE(event, bagID)
	-- update the item info
	addon:update()
end

function addon:BANKFRAME_OPENED()
	-- set the bank frame flag and update the item info
	bankFrameOpen = true
	addon:update()
end

function addon:BANKFRAME_CLOSED()
	-- clear the bank frame flag
	bankFrameOpen = false
end

function addon:OnDragStart(self, button)
	-- check to see if this is the minimap button and the right button
	if ((self == repFrames.minimapButton) and (button == "RightButton")) then
		-- it is so start moving it
		--GameTooltip:Hide()
		buttonMoving = true
		--self:StartMoving()
	end
end

function addon:OnDragStop(self)
	-- check to see if this the minimap button
	if (self == repFrames.minimapButton) then
		-- it is so lets stop moving it
		--self:StopMovingOrSizing()
		buttonMoving = false
	end
end

function addon:OnUpdate(self, elepsed)
	-- check to see if this is the minimap button and it is moving
	if ((self == repFrames.minimapButton) and buttonMoving) then
		-- it is so lets move the button
		addon:UpdateButtonSetting()
	end
	
	if (needToUpdate) then
		needToUpdate = false
		addon:update()
	end
end

function addon:OnMouseUp(self, button)
	if ((self == repFrames.anchor) and (button == "RightButton")) then
		-- the main frame and right button has been pressed
		-- show the floating menu for the frame as an easy menu
		EasyMenu(addon.menuOptions, repFrames.dropDownMenu, "cursor", 0, 0, "MENU", 1)
	elseif ((self == repFrames.minimapButton) and (button == "LeftButton")) then
		-- the minimap button and right button has been pressed
		addon.db.profile.frameClosedByUser = not addon.db.profile.frameClosedByUser
		addon:ShowFrame(true)
		AceConfigReg:NotifyChange(modName)
	end
end

function addon:OnEnter(self, motion)
	if ((self == repFrames.minimapButton) and (not buttonMoving)) then
		GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
		GameTooltip:SetText("|cffffffff"..L["Rep Items"])
		GameTooltip:AddLine(L["Left-click to toggle window"])
		GameTooltip:AddLine(L["Right-click for drag"])
		GameTooltip:Show()
	elseif (self == repFrames.query) then
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 0, -10)
		addon:ShowTotalReputations()
		GameTooltip:Show()
	else
		-- and if there is data in the link to display
		if (self.link) then
			-- show the tooltip menu
			GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
			addon:SetTooltipMessage(self.link)
			GameTooltip:Show()
		end
	end
end

function addon:OnLeave()
	-- hide whatever tooltip message is shown
	GameTooltip:Hide()
end

function addon:OnClick(self, button, down)
	-- make sure that the click is from the close box
	if (self == repFrames.close) then
		-- save the state in the global variable and notify the interface frame
		addon.db.profile.frameClosedByUser = true
		AceConfigReg:NotifyChange(modName)
	end
end

function addon:OnTooltipSetItem(self)
	-- only do this if the lineAdded flag is enabled, this is the safe way to do it (apparently)
	if (not lineAdded) then
		-- set the tooltip flag
		lineAdded = true
		
		-- only show if the tooltip flag is enabled
		if (addon.db.profile.ShowTooltips) then
			-- get the item id
			local name, link = self:GetItem()
			-- if there is a name then it probably carried by the user otherwise it is probably from a trainer
			if (name) then
				local id = tonumber(link:match("|Hitem:(%d+):"))
				-- check to see if the id is in the items list
				if (addon.repItems[id]) then
					-- it is so lets add it to the tooltip for each reputation
					self:AddLine("|cffffffff"..L["Used in reputation gain with"])
					for idx, value in pairs(addon.repItems[id]["entry"]) do
						-- make sure that the faction is correct for the player
						if ((addon.reputations[value["reputation"]]["faction"] == "") or (addon.reputations[value["reputation"]]["faction"] == addon.db.profile.faction)) then
							-- add the reputation message
							self:AddLine(" \""..addon.reputations[value["reputation"]]["text"].."\"")
						end
					end
				end
			end
		end
	end
end

function addon:OnTooltipCleared(self)
	-- clear the lineAdded line add flag if it is enabled
	if (lineAdded) then
		lineAdded = false
	end
end

-------------------------------
-- handle the slash commands --
-------------------------------
function addon:SlashCommandHandler(input)
    if(input == "show") then
		-- clear the show state and show the frame
		addon.db.profile.frameClosedByUser = false
        addon:ShowFrame(true)
        addon:update()
    elseif (input == "hide") then
		-- set the show state and hide the frame
		addon.db.profile.frameClosedByUser = true
        addon:ShowFrame(false)
    elseif (input == "config") then
		-- show the config page
		addon:OpenInterfacePage()
    elseif (input == "reset") then
		-- reset the frame
		addon:ResetFrame()
    end
    
    -- update the interface pannel
	AceConfigReg:NotifyChange(modName)
end

--[[*******************
	* Local functions *
	*****************]]
	
----------------------------------------
-- update the minimap button position --
----------------------------------------
function addon:UpdateButtonSetting()
	-- this came from Atlas: AtlasButton.lua
	-- I combined  the functions in to one routine for my use.
	
	local xpos, ypos = GetCursorPosition()
	local xmin, ymin = Minimap:GetLeft(), Minimap:GetBottom()
	
	xpos = xmin - xpos / UIParent:GetScale() + 70
	ypos = ypos / UIParent:GetScale() - ymin - 70
	
	addon.db.profile.MinimapButtonPosition = math.floor( math.deg(math.atan2(ypos, xpos)))
	
	if (addon.db.profile.MinimapButtonPosition < 0) then
		addon.db.profile.MinimapButtonPosition = addon.db.profile.MinimapButtonPosition + 360
	end
	
	addon:UpdateButtonPosition()
end

--------------------------------------------
-- Reposition the button to the set value --
--------------------------------------------
function addon:UpdateButtonPosition()
	repFrames.minimapButtonFrame:SetPoint("TOPLEFT", "Minimap", "TOPLEFT", 54 - (80 * cos(addon.db.profile.MinimapButtonPosition)), (80 * sin(addon.db.profile.MinimapButtonPosition)) - 54)
	AceConfigReg:NotifyChange(modName)
end

---------------------------
-- Toggle minimap button --
---------------------------
function addon:ToggleButton()
	if (addon.db.profile.ShowMinimapButton) then
		repFrames.minimapButtonFrame:Show()
	else
		repFrames.minimapButtonFrame:Hide()
	end
	
end

-------------------------------------
-- Open the interface options page --
-------------------------------------
function addon:OpenInterfacePage()
	InterfaceOptionsFrame_OpenToCategory(repFrames.interfacePage)
end

--------------------------------------------
-- Reset the addon back to default values --
--------------------------------------------
function addon:ResetToDefaults()
	self.db:ResetProfile()
	AceConfigReg:NotifyChange(modName)
	addon:ResetFrame()
	addon:update()
	addon:UpdateButtonPosition()
end

----------------------------------------------
-- Reset the frame back to default position --
----------------------------------------------
function addon:ResetFrame()
	-- clear the show state, reset the frame position and show it
	addon.db.profile.frameClosedByUser = false
	addon:ShowFrame(true)
	repFrames.anchor:ClearAllPoints()
	repFrames.anchor:SetPoint("TOPLEFT", PlayerFrame, "BOTTOMLEFT", 10, 10)	
end

----------------------
-- main update loop --
----------------------
function addon:update()
	addon:GetItemsInBags()
	addon:CleanUpItemsInBags()
	addon:UpdateFrame()
end

---------------------------
-- update just the frame --
---------------------------
function addon:UpdateFrame()
	addon:CreateItemButtons()
	addon:ResizeFrame()
end

---------------------------------------
-- Get all the rep items in the bags --
---------------------------------------
function addon:GetItemsInBags()
	-- this just loops through all the available bags and ignores those that dont exist
	-- not sure about the speed benifits of this feature
    
    -- use just the bag slots and key ring
    local bagStart = -2
    local bagEnd = 12

	-- clear the item count for the bags
	for itemID,_ in pairs(addon.db.profile.itemsInBags) do
		-- clear the local one first
		addon.db.profile.itemsInBags[itemID]["bags"] = 0
		addon.db.profile.itemsInBags[itemID]["keyring"] = 0
		-- if the bank is open then clear that as well
		if (bankFrameOpen) then
			addon.db.profile.itemsInBags[itemID]["bank"] = 0
		end
	end
    
    -- start looping though the bags using the id as the index
    for bagID = bagStart, bagEnd do
        -- get the bag name as they will all have one except the original bank tab(-1) which is nil
        local bagName = GetBagName(bagID)
        -- get the number of slots in the bag and loop if there are slots
		for slotID = 1, GetContainerNumSlots(bagID) do
			-- get the id of the item in the bag slot, nil means that it is empty
            local itemID = GetContainerItemID(bagID, slotID)
            if(itemID ~= nil) then
                -- check to see if this item is a rep item, nil means no
                if(addon.repItems[itemID] ~= nil) then
                    -- get the item info, like the number of them
                    local texture, count, locked, quality, readable, lootable, link = GetContainerItemInfo(bagID, slotID)
                    -- check to see if there is an item in the disply list, if no then add it for the next bit
                    if (addon.db.profile.itemsInBags[itemID] == nil) then
                        addon.db.profile.itemsInBags[itemID] = {["bags"] = 0, ["bank"] = 0, ["keyring"] = 0, ["link"] = link, ["texture"] = texture, }
                    end
                    
                    -- this is to check in which list to add the count of the item, bags or bank
                    if (bagID == -2) then
						addon.db.profile.itemsInBags[itemID]["keyring"] = addon.db.profile.itemsInBags[itemID]["keyring"] + count
					elseif (bagID > 4 or bagID < 0) then
                        addon.db.profile.itemsInBags[itemID]["bank"] = addon.db.profile.itemsInBags[itemID]["bank"] + count
                    else
                        addon.db.profile.itemsInBags[itemID]["bags"] = addon.db.profile.itemsInBags[itemID]["bags"] + count
                    end
                end
            end
        end
    end
end 

------------------------------------------------------------------------
-- Removes any item that has no count or does not exist as a rep item --
------------------------------------------------------------------------
function addon:CleanUpItemsInBags()
	-- just loop through the items that the user has
	for idx,_ in pairs(addon.db.profile.itemsInBags) do
		-- is the item count = 0 or the item has been removed from the rep list
		if ((addon:GetItemInBagsCount(idx) == 0) or (not addon.repItems[idx])) then
			-- it is so just remove the item from the list
			addon.db.profile.itemsInBags[idx] = nil
		end
	end
end

-------------------------------------------------
-- Create item buttons to be used in the frame --
-------------------------------------------------
function addon:CreateItemButtons()
	-- loop through the list of items on hand to create buttons if needed
	local idx = 0
	
	-- loop through each item that the toon has on them
	for id,item in pairs(addon.db.profile.itemsInBags) do
		if (item["keyring"] == nil) then item["keyring"] = 0 end
		if (item["bags"] == nil) then item["bags"] = 0 end
		if (item["bank"] == nil) then item["bags"] = 0 end
		
		if ((item["keyring"] > 0) or (item["bags"] > 0) or ((item["bank"] > 0) and (addon.db.profile.ShowBankItems))) then
			-- make sure that the item exists first
			if (addon.repItems[id]) then
				-- see if we need to work on this item or not
				if ((addon.repItems[id]["major"] == true) or ((addon.repItems[id]["major"] == false) and (addon.db.profile.ShowMinorItems == true))) then
					-- update the button index and add a new one if it does not exist
					idx = idx + 1
					if (idx > #repItemButtons) then
						addon:AddButton()
					end
					
					-- work out the value that is shown on the button for the user
					local countText = ""
					local stockText = ""
					if (addon.db.profile.ShownValue == "QUANTITY") then
						countText = item["bags"] + item["bank"] + item["keyring"]
					elseif (addon.db.profile.ShownValue == "HANDINS") then
						-- check to see if this is a major item
						if (addon.repItems[id]["major"]) then
							-- just assume that the handin ammount is the same for all the reps					
							countText = math.floor((item["bags"] + item["bank"] + item["keyring"]) / addon.repItems[id]["entry"][1]["quantity"] * 10) / 10
						else
							-- it is not a major item so that means that it is part of another handin so lets just ignore this one
							countText = ""
						end
					end
					
					-- get the next button to work on and place all the item info on it
					local button = repItemButtons[idx] 
					button:SetAttribute("item2", item["link"])
					button.link = id
					button.icon:SetTexture(item["texture"])
					button.count:SetText(countText)
					button.count:Show()
					button:Show()
				end
			end
		end
	end

	-- there may be buttons that have been created that do not contain anything anymore 
	-- so loop through the remainder to zero and hide any that are showing
	for unusedIdx = idx + 1, #repItemButtons do
		repItemButtons[unusedIdx].link = 0
		repItemButtons[unusedIdx]:Hide()
	end
end

-------------------------------------------------------------------------------------------------
-- Adds the quantities that may be on other character to the tooltip message that is displayed --
-------------------------------------------------------------------------------------------------
function addon:AddQuantitiesTooltipMessage(id)
	-- get the item information
	local itemName = addon:GetItemNameFromID(id)

	-- display the name of the item and the number of them in the bags and bank
	GameTooltip:SetText("|cffffffff"..itemName)
	GameTooltip:AddLine(L["bags bank items"](playerName, addon.db.profile.itemsInBags[id]["bags"], addon.db.profile.itemsInBags[id]["bank"], addon.db.profile.itemsInBags[id]["keyring"]))
	
	-- check to see if the user wants to see items from the other characters and its not "BOP"
	if ((addon.db.profile.ShowItemsFromOtherCharacters) and (not addon.repItems[id]["bop"])) then
		-- get the other character names
		local otherCharacters = {}
		addon.db:GetProfiles(otherCharacters)
		
		-- loop though each of the characters that are available
		for _, characterName in pairs(otherCharacters) do
			-- split the name in to name and realm
			local splitName = { strsplit(" ", characterName)}

			-- check that the player is not this name
			if (splitName[1] ~= playerName) then
				-- check that the realm is the same
				if (splitName[3] == realmName) then
					-- and make sure that the faction is the same
					if (RepItemsDB["profiles"][characterName]["faction"] == addon.db.profile.faction) then
						-- make sure that there is a variable to loop through first
						if (RepItemsDB["profiles"][characterName]["itemsInBags"]) then
							-- loop through each character and display the item count
							for itemID, item in pairs(RepItemsDB["profiles"][characterName]["itemsInBags"]) do
								-- check to see if this is the same item as the one we are looking at
								if (itemID == id) then
									-- now we can add the line for the other character
									GameTooltip:AddLine(L["bags bank items"](splitName[1], item["bags"], item["bank"], item["keyring"]))
								end
							end
						end
					end
				end
			end
		end
	end
end

------------------------------------------
-- Returns the rep amount based on race --
------------------------------------------
function addon:GetRepRaceAmount(amount)
	local changedAmount = amount + math.ceil(amount * repMultiplier / 100)
	return changedAmount
end

-------------------------------------------------------------
-- Sets the tool tip message based on the supplied item id --
-------------------------------------------------------------
function addon:SetTooltipMessage(id)
	-- add the quantity tooltip message
	addon:AddQuantitiesTooltipMessage(id)
	
	-- loop through each rep that the hand in is used for
	for _,repEntry in pairs(addon.repItems[id]["entry"]) do
		-- get the item faction
		local itemFaction = addon.reputations[repEntry["reputation"]]["faction"]
		-- work out how much of a hand in the item is
		if ((itemFaction == "") or (itemFaction == addon.db.profile.faction)) then
			-- check to see if this is an extra item
			if (addon.repItems[id]["major"] == false) then
				GameTooltip:AddLine(L["Used for rep hand-ins with"])
				-- loop through all the availible rep handins
				for _,repName in pairs(addon.repItems[id]["entry"]) do
					GameTooltip:AddLine(L["rep name"](addon.reputations[repName["reputation"]]["text"]))
				end
			else
				-- check to see if there are no additional requirments for the hand in
				if (not repEntry["additional"]) then
					-- work out how many hand ins there are and and much they give
					local numberOfHandins = math.floor(addon:GetItemInBagsCount(id) / repEntry["quantity"])
					local amountOfRep = addon:GetRepRaceAmount(repEntry["handin"] * numberOfHandins)
					
					-- create the message that is shown
					local repOwner = L["rep name"](addon.reputations[repEntry["reputation"]]["text"])
					local repAmount = L["repAmount"](amountOfRep)
					if (amountOfRep > 0) then
						repAmount = repAmount..L["repAmount handins"](numberOfHandins)
					end
					
					GameTooltip:AddDoubleLine(repOwner, repAmount)
				else
					-- show the owner of the reputation
					GameTooltip:AddLine(L["rep name"](addon.reputations[repEntry["reputation"]]["text"]))
					
					-- work through each additional hand in set
					for additionalEntryIdx,additionalEntry in pairs(repEntry["additional"]) do
						if (additionalEntryIdx > 1) then
							GameTooltip:AddLine(" ")
						end
						
						local numberOfHandins = 0
						local amountOfRep = 0
						
						-- work out the number of hand ins if there is a quantity
						numberOfHandins = math.floor(addon:GetItemInBagsCount(id) / repEntry["quantity"])
						
						-- work through each set 
						for entryIdx,entry in pairs(additionalEntry) do
							local name = addon:GetItemNameFromID(entry["id"])
							local numberOfItems = addon:GetItemInBagsCount(entry["id"])
							local requiredItems = "|cffffffff"..entry["quantity"].."|cffffD100 x "..name
							if (entry["vendor"] == true) then
								requiredItems = requiredItems..L["vendor"]
							end
							local haveItems = L["have items"](numberOfItems)
							GameTooltip:AddDoubleLine(requiredItems, haveItems)
							
							local num = math.floor(numberOfItems / entry["quantity"])
							if ((num < numberOfHandins) and (entry["vendor"] == false)) then
								numberOfHandins = num
							end
						end
						
						amountOfRep = addon:GetRepRaceAmount(repEntry["handin"]) * numberOfHandins
						-- create the message that is shown
						local repOwner = L["rep name"](addon.reputations[repEntry["reputation"]]["text"])
						local repAmount = L["repAmount"](amountOfRep)
						if (amountOfRep > 0) then
							repAmount = repAmount..L["repAmount handins"](numberOfHandins)
						end
						GameTooltip:AddDoubleLine(" ", repAmount)
					end
				end
			end
		end
	end
end

-------------------------------------------------------------
-- Get the item from the id                                --
--   If it is not in cache then look in the local database --
--   If not there then return just the id message          --
-------------------------------------------------------------
function addon:GetItemNameFromID(id)
	-- get the item name from the local cache
	local name = select(1, GetItemInfo(id)) 
	-- check to see if there was a name returned
	if (not name) then
		-- there was not so it is not in the cache file
		-- check to see if it is the local database
		if (addon.repItems[id]) then
			-- it is so send back the information
			name = addon.repItems[id]["text"]
		else
			-- not there so send a message
			name = "Item id #"..id
		end
	end
	
	-- return the name
	return name
end
			
------------------------------------------------------------
-- Get the total count for the item id in the player bags --
------------------------------------------------------------
function addon:GetItemInBagsCount(id)
	-- default value is 0
	local itemCount = 0
	
	-- check to see if the user is carrying that item
	if (addon.db.profile.itemsInBags[id]) then
		-- they are so just all of them up
		itemCount = itemCount + addon.db.profile.itemsInBags[id]["bags"] + addon.db.profile.itemsInBags[id]["bank"] + addon.db.profile.itemsInBags[id]["keyring"]
	end
	
	-- return the value
	return itemCount
end

-------------------------------------
-- add another button to the array --
-------------------------------------
function addon:AddButton()
	-- work out the next button index
	local nextButtonID = #repItemButtons + 1
	
	-- create a new button using the template
	local item = CreateFrame("Button", "$parent_"..nextButtonID, repFrames.anchor, repFrames.template)
	--item:RegisterForClicks("RightButtonUp")
	addon:HookScript(item, "OnEnter")
	addon:HookScript(item, "OnLeave")
	
	-- store the new button in the array
	repItemButtons[nextButtonID] = item
end

---------------------------------------------------------------------
-- Resize the frame to hold all the available items that are shown --
---------------------------------------------------------------------
function addon:ResizeFrame()
	local totalVisibleButtons = 0
	
	-- loop through the item buttons
	for idx = 1, #repItemButtons do
		-- make sure that the button has something to show
		if (repItemButtons[idx].link > 0) then 
			-- set up the location depending on where the button is based on the column
			if (totalVisibleButtons == 0) then
				-- there are no items to show
				repItemButtons[1]:SetPoint("TOPLEFT", 8, -35)
			elseif (addon.db.profile.numberOfColumns == 1) then
				-- there is only 1 column so all the items will show below the previous one
				repItemButtons[idx]:SetPoint("TOPLEFT", repItemButtons[idx-1], "BOTTOMLEFT", 0, -4)
			elseif (math.fmod(idx, addon.db.profile.numberOfColumns) == 1) then
				-- this is the first in the column so allign it with the previous first one
				repItemButtons[idx]:SetPoint("TOPLEFT", repItemButtons[idx - addon.db.profile.numberOfColumns], "TOPLEFT", 0, -frameIncreaseSize)	
			else
				-- allign this next to the previous one
				repItemButtons[idx]:SetPoint("TOPLEFT", repItemButtons[idx - 1], "TOPLEFT", frameIncreaseSize, 0)
			end
			
			-- increase the visible button count
			totalVisibleButtons = totalVisibleButtons + 1
		end
	end
	
	-- set up frame size and visibility
	if (totalVisibleButtons < 1) then
		-- there are no items so just create the smallest frame posible
		repFrames.anchor:SetHeight(38)
		repFrames.anchor:SetWidth(frameIncreaseSize + 9)
		
		addon:ShowFrame(not addon.db.profile.ShowOnlyWhenThereAreItems)
	else
		-- there are items so work out what size the frame should be
		repFrames.anchor:SetHeight(frameIncreaseSize * math.ceil(totalVisibleButtons / addon.db.profile.numberOfColumns) + 38)
		if (totalVisibleButtons >= addon.db.profile.numberOfColumns) then
			repFrames.anchor:SetWidth(frameIncreaseSize * addon.db.profile.numberOfColumns + 9)
		else
			repFrames.anchor:SetWidth(frameIncreaseSize * totalVisibleButtons + 9)
		end
		
		addon:ShowFrame(true)
	end
		
	-- set the scale
	repFrames.anchor:SetScale(addon.db.profile.UIScale)
end

------------------------------------------
-- Show the frame depending on settings --
------------------------------------------
function addon:ShowFrame(showIt)
	-- now work out which way to show or hide
	if ((showIt and (not addon.db.profile.frameClosedByUser)) or bankFrameOpen) then
		repFrames.anchor:Show()
	else
		repFrames.anchor:Hide()
	end
end

------------------------------------------------------------------------------
-- Adds the total rep values for all the items in the database to a tooltip --
------------------------------------------------------------------------------
function addon:ShowTotalReputations()
	local reputations = {}	-- holds the list of reputations found
	local itemsHeld = {}	-- holds the list of items that have already been counted. This avoids duplicate counts
	
	-- get the other character names
	local otherCharacters = {}
	addon.db:GetProfiles(otherCharacters)
	
	-- check to see if the user wants to see other characters items
	if (addon.db.profile.ShowItemsFromOtherCharacters) then
		-- loop though each of the characters that are available
		for _, characterName in pairs(otherCharacters) do
			-- split the name in to name and realm
			local splitName = { strsplit(" ", characterName)}

			-- check that the player is not this name
			if (splitName[1] ~= playerName) then
				-- check that the realm is the same
				if (splitName[3] == realmName) then
					-- and make sure that the faction is the same
					if (RepItemsDB["profiles"][characterName]["faction"] == addon.db.profile.faction) then
						-- make sure that there is a variable to loop through first
						if (RepItemsDB["profiles"][characterName]["itemsInBags"]) then
							-- loop through each character and add the item count
							for itemID, item in pairs(RepItemsDB["profiles"][characterName]["itemsInBags"]) do
								-- only count this if it is not BOP
								if (not addon.repItems[itemID]["bop"]) then
									-- check to see if this has been created before and if not create a blank entry
									if (itemsHeld[itemID] == nil) then
										itemsHeld[itemID] = {["counted"] = false, ["count"] = 0, }
									end
									
									-- add the item to the list
									itemsHeld[itemID]["count"] = itemsHeld[itemID]["count"] + item["bags"] + item["bank"] + item["keyring"]
								end
							end
						end
					end
				end
			end
		end
	end
		
	-- loop through all the rep items that are in bags
	for itemID, item in pairs(addon.db.profile.itemsInBags) do
		-- check to see if this has been created before and if not create a blank entry
		if (itemsHeld[itemID] == nil) then
			itemsHeld[itemID] = {["counted"] = false, ["count"] = 0, }
		end
		
		-- add the item to the list
		itemsHeld[itemID]["count"] = itemsHeld[itemID]["count"] + item["bags"] + item["bank"] + item["keyring"]
	end
	
	-- now create a list of reputations from the items that the player has
	for itemID,item in pairs(itemsHeld) do
		-- check to see if we have seen this before
		if (not item["counted"]) then
			-- we have not so lets count it
			item["counted"] = true
			-- only work on this item if it is a major item
			if (addon.repItems[itemID]["major"]) then
				-- loop through each entry for the item
				for entryIdx,entry in pairs(addon.repItems[itemID]["entry"]) do
					-- check to see if this item is used for the players faction
					if ((addon.reputations[entry["reputation"]]["faction"] == "") or (addon.reputations[entry["reputation"]]["faction"] == addon.db.profile.faction)) then
						-- if there is no entry for the reputation then create a blank one
						if (reputations[entry["reputation"]] == nil) then
							reputations[entry["reputation"]] = { ["gain"] = 0, }
						end
						-- add the reputation gain
						
						-- only work on this if there is at least 1 handin
						if (math.floor(item["count"] / entry["quantity"]) > 0) then
							local gain = math.floor(item["count"] / entry["quantity"]) * entry["handin"]
							-- check to see if this not is a single item
							if (entry["additional"]) then
								local additionalCount = 0
								-- loop through each of the extra combinations
								for _,additionalEntry in pairs(entry["additional"]) do
									-- create the maximum number of handins for this combination
									local combinationCount = math.floor(item["count"] / entry["quantity"])
									-- loop through each entry of the combination
									for extraIdx,extra in pairs(additionalEntry) do 
										-- check to see that it is not a vendor item, since with a vendor item you can buy as many as you need
										if(not extra["vendor"]) then
											-- see if there is the required extra item first
											if (itemsHeld[extra["id"]]) then
												itemsHeld[extra["id"]]["counted"] = true
												
												-- work out the maximum number of handins for that item
												local extraCount = math.floor(itemsHeld[extra["id"]]["count"] / extra["quantity"])
												if (extraCount < combinationCount) then
													combinationCount = extraCount
												end
											end
										end
									end
									
									-- check to see if there is a greater handin value
									if (additionalCount < combinationCount) then
										additionalCount = combinationCount
									end
								end
								gain = additionalCount * entry["handin"]
							end
							reputations[entry["reputation"]]["gain"] = reputations[entry["reputation"]]["gain"] + addon:GetRepRaceAmount(gain)
						end
					end
				end
			end
		end
	end
	
	-- show the reputations to the user
	GameTooltip:SetText("|cffffffff"..L["Summary of Rep Gains"])
	for idx,entry in pairs(reputations) do
		if (entry["gain"] > 0) then
			GameTooltip:AddDoubleLine(addon.reputations[idx]["text"], "|cffffffff"..math.floor(entry["gain"]))
		end
	end
end
